require 'csv'
require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
	class Game
		def initialize(title)
			@title = title
			@players = []
		end

		def save_high_scores(to_file="high_scores.txt")
			File.open(to_file,'w') do |file|
				file.puts "\n#{@title} High Scores:"	
				@players.sort.each do |player|
					file.puts high_score_entry(player)
				end
			end
		end

		def high_score_entry(player)
			formatted_name = player.name.ljust(20,".")
			"#{formatted_name} #{player.score}"
		end 

		def load_players(from_file)
			CSV.foreach(from_file) do |row|
				player = Player.new(row[0], Integer(row[1]))
				add_player(player)
			end

			#use the below method to make use of the 'from_csv' method in player.rb
			
			# File.readlines(from_file).each do |line|
			# 	add_player(Player.from_csv(line))
			# end
		end

		def add_player(a_player)
			@players << a_player
		end

		def play(rounds)
			puts "There are #{@players.size} players in #{@title}:"
			@players.each do |player|
				puts player
			end

			treasures = TreasureTrove::TREASURES

			puts "\nThere are #{treasures.length} treasures to be found:"
			treasures.each do |treasure|
			puts "A #{treasure.name} is worth #{treasure.points} points"
			end 

			1.upto(rounds) do |round|

				if block_given?
					break if yield
				end

				puts "\nRound number #{round}:"
				@players.each do |player|
					GameTurn.take_turn(player)
				end
			end
		end

		def print_health_and_name(player)
			puts "#{player.name} (#{player.health})"
		end
		
		def total_points
			@players.reduce(0) { |sum, player| sum + player.points }
		end
		
		def print_stats
			strong_players, wimpy_players = @players.partition { |p| p.strong? }
			puts "\n#{@title} Statistics:"  
			puts "\n#{strong_players.length} strong players:"
			strong_players.each do |player|
				print_health_and_name(player)
			end
			puts "\n#{wimpy_players.length} wimpy players:"
			wimpy_players.each do |player| 
				print_health_and_name(player)
			end

			puts "\n#{@title} High Scores:"	
			@players.sort.each do |player|
			puts high_score_entry(player)
			end

			@players.each do |player|
				puts "\n#{player.name}'s point totals:"
				puts "#{player.score} grand total points"
			end

			@players.sort.each do |player|
				puts "\n#{player.name}'s point totals"
				player.each_found_treasure do |treasure|
				puts "#{treasure.points} total #{treasure.name} points"
				end
				puts "#{player.points} grand total points"
			end

			puts "#{total_points} total points from treasures found"
		end

	end
end 
